#!/bin/bash

# Copyright (C) 2017 Red Hat, Inc.
# Written by Andrew John Hughes <gnu.andrew@redhat.com>.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

GENERATED_SRC=${1}
GENERATED_BUILD=${2}
REPO_DIR=${3}

if test "x${GENERATED_SRC}" = x; then
    echo "Need to specify generated source directory.";
    echo "${0} <GENERATED_SRC> <GENERATED_BUILD> <REPO_DIR>"
    exit 1;
fi

if test "x${GENERATED_BUILD}" = x; then
    echo "Need to specify generated build directory.";
    echo "${0} <GENERATED_SRC> <GENERATED_BUILD> <REPO_DIR>"
    exit 2;
fi

if test "x${REPO_DIR}" = x; then
    REPO_DIR=${GENERATED_SRC}/..;
    echo "Assuming repository is directory above generated source directory; ${REPO_DIR}";
    if ! hg tip -R ${REPO_DIR}; then
	echo "But doesn't appear to be a repository.";
	exit 3;
    fi
fi

echo "Generated source directory: ${GENERATED_SRC}"
echo "Generated build directory: ${GENERATED_BUILD}"

pushd ${GENERATED_SRC}
for files in `find . -type f`;
do
    echo "Copying latest version of ${files}";
    'cp' ${GENERATED_BUILD}/${files} ${files};
done

for dirs in `find . -type d`;
do
    for files in `(cd ${GENERATED_BUILD}/ ; find ${dirs} -maxdepth 1 -type f )`;
    do
	echo "Does ${GENERATED_SRC}/${files} exist?"
	if [ ! -e ${GENERATED_SRC}/${files} ]; then
	    echo "No, copying ${files} to ${GENERATED_SRC}";
	    cp ${GENERATED_BUILD}/${files} ${GENERATED_SRC}/${files} ;
	    hg add -R ${REPO_DIR} ${GENERATED_SRC}/${files};
	else
	    echo "Yes, skip that one."
	fi;
    done;
done

new_count=$(hg status -R ${REPO_DIR}|grep '^A'|wc -l)
mod_count=$(hg status -R ${REPO_DIR}|grep '^M'|wc -l)
echo "${new_count} new files added. ${mod_count} old files updated."
